@@ -73,8 +73,20 @@
 #'   be considered in the stability score. The use of
 #'   \code{group_penalisation=TRUE} strictly applies to group (not sparse-group)
 #'   penalisation.
-#' @param n_cores number of cores to use for parallel computing (see
-#'   \code{\link[parallel]{mclapply}}). Only available on Unix systems.
+#' @param optimisation character string indicating the type of optimisation
+#'   method. With \code{optimisation="grid_search"} (the default), all values in
+#'   \code{Lambda} are visited. Alternatively, optimisation algorithms
+#'   implemented in \code{\link[nloptr]{nloptr}} can be used with
+#'   \code{optimisation="nloptr"}. By default, we use
+#'   \code{"algorithm"="NLOPT_GN_DIRECT_L"}, \code{"xtol_abs"=0.1},
+#'   \code{"ftol_abs"=0.1} and \code{"maxeval"=Lambda_cardinal}. These values
+#'   can be changed by providing the argument \code{opts} (see
+#'   \code{\link[nloptr]{nloptr}}). For stability selection using penalised
+#'   regression, \code{optimisation="grid_search"} may be faster as it allows
+#'   for warm start.
+#' @param n_cores number of cores to use for parallel computing (see argument
+#'   \code{workers} in \code{\link[future]{multisession}}). Using
+#'   \code{n_cores>1} is only supported with \code{optimisation="grid_search"}.
 #' @param output_data logical indicating if the input datasets \code{xdata} and
 #'   \code{ydata} should be included in the output.
 #' @param verbose logical indicating if a loading bar and messages should be
@@ -144,8 +156,8 @@
 #'   number generator is set to \code{seed}.
 #'
 #'   For parallelisation, stability selection with different sets of parameters
-#'   can be run on \code{n_cores} cores. This relies on forking with
-#'   \code{\link[parallel]{mclapply}} (specific to Unix systems). Alternatively,
+#'   can be run on \code{n_cores} cores. Using \code{n_cores > 1} creates a
+#'   \code{\link[future]{multisession}}. Alternatively,
 #'   the function can be run manually with different \code{seed}s and all other
 #'   parameters equal. The results can then be combined using
 #'   \code{\link{Combine}}.
@@ -280,7 +292,7 @@
 #'   simul <- SimulateComponents(pk = c(5, 3, 4))
 #'   stab <- VariableSelection(
 #'     xdata = simul$data,
-#'     Lambda = 1:(ncol(simul$data) - 1),
+#'     Lambda = seq_len(ncol(simul$data) - 1),
 #'     implementation = SparsePCA
 #'   )
 #'   CalibrationPlot(stab, xlab = "")
@@ -293,7 +305,7 @@
 #'   simul <- SimulateRegression(n = 100, pk = 50, family = "gaussian")
 #'   stab <- VariableSelection(
 #'     xdata = simul$xdata, ydata = simul$ydata,
-#'     Lambda = 1:(ncol(simul$xdata) - 1),
+#'     Lambda = seq_len(ncol(simul$xdata) - 1),
 #'     implementation = SparsePLS, family = "gaussian"
 #'   )
 #'   CalibrationPlot(stab, xlab = "")
@@ -304,7 +316,7 @@
 #' if (requireNamespace("sgPLS", quietly = TRUE)) {
 #'   stab <- VariableSelection(
 #'     xdata = simul$xdata, ydata = simul$ydata,
-#'     Lambda = 1:5,
+#'     Lambda = seq_len(5),
 #'     group_x = c(5, 5, 10, 20, 10),
 #'     group_penalisation = TRUE,
 #'     implementation = GroupPLS, family = "gaussian"
@@ -347,7 +359,7 @@
 #'   simul <- SimulateRegression(n = 200, pk = 20, family = "binomial")
 #'   ManualGridGroupLasso <- function(xdata, ydata, family, group_x, ...) {
 #'     # Defining the grouping
-#'     group <- do.call(c, lapply(1:length(group_x), FUN = function(i) {
+#'     group <- do.call(c, lapply(seq_len(length(group_x)), FUN = function(i) {
 #'       rep(i, group_x[i])
 #'     }))
 #'
@@ -368,7 +380,7 @@
 #'   )
 #'   GroupLasso <- function(xdata, ydata, Lambda, family, group_x, ...) {
 #'     # Defining the grouping
-#'     group <- do.call(c, lapply(1:length(group_x), FUN = function(i) {
+#'     group <- do.call(c, lapply(seq_len(length(group_x)), FUN = function(i) {
 #'       rep(i, group_x[i])
 #'     }))
 #'
@@ -409,15 +421,23 @@ VariableSelection <- function(xdata, ydata = NULL, Lambda = NULL, pi_list = seq(
                               resampling = "subsampling", cpss = FALSE,
                               PFER_method = "MB", PFER_thr = Inf, FDP_thr = Inf,
                               Lambda_cardinal = 100, group_x = NULL, group_penalisation = FALSE,
+                              optimisation = c("grid_search", "nloptr"),
                               n_cores = 1, output_data = FALSE, verbose = TRUE, beep = NULL, ...) {
-  # Defining Lambda if used with sparse PCA or PLS
   if (is.null(Lambda)) {
+    # Defining Lambda if used with sparse PLS
     if (as.character(substitute(implementation)) %in% c("SparseGroupPLS", "GroupPLS")) {
       Lambda <- seq(1, length(group_x) - 1)
     }
+
+    # Defining Lambda if used with sparse PCA
     if (as.character(substitute(implementation)) %in% c("SparsePLS", "SparsePCA")) {
       Lambda <- seq(1, ncol(xdata) - 1)
     }
+
+    # Defining Lambda if used with CART
+    if (as.character(substitute(implementation)) %in% c("CART")) {
+      Lambda <- seq(1, min(nrow(xdata) / 2, 100))
+    }
   }
 
   # Object preparation, error and warning messages
@@ -451,32 +471,103 @@ VariableSelection <- function(xdata, ydata = NULL, Lambda = NULL, pi_list = seq(
     )
   }
 
-  # Check if parallelisation is possible (forking)
-  if (.Platform$OS.type != "unix") {
-    if (n_cores > 1) {
-      warning("Invalid input for argument 'n_cores'. Parallelisation relies on forking, it is only available on Unix systems.")
-    }
-    n_cores <- 1
-  }
+  # Defining the type of optimisation
+  optimisation <- match.arg(optimisation)
+
+  # Storing extra arguments
+  extra_args <- list(...)
 
   # Stability selection and score
-  mypar <- parallel::mclapply(X = 1:n_cores, FUN = function(k) {
-    return(SerialRegression(
-      xdata = xdata, ydata = ydata, Lambda = Lambda, pi_list = pi_list,
-      K = ceiling(K / n_cores), tau = tau, seed = as.numeric(paste0(seed, k)), n_cat = n_cat,
-      family = family, implementation = implementation,
-      resampling = resampling, cpss = cpss,
-      PFER_method = PFER_method, PFER_thr = PFER_thr, FDP_thr = FDP_thr,
-      group_x = group_x, group_penalisation = group_penalisation,
-      output_data = output_data, verbose = verbose, ...
-    ))
-  })
-
-  # Combining the outputs from parallel iterations
-  out <- mypar[[1]]
   if (n_cores > 1) {
-    for (i in 2:length(mypar)) {
-      out <- do.call(Combine, list(stability1 = out, stability2 = mypar[[i]]))
+    if (optimisation != "grid_search") {
+      message("Using grid search to allow for parallelisation.")
+    }
+    future::plan(future::multisession, workers = n_cores)
+    mypar <- future.apply::future_lapply(X = seq_len(n_cores), future.seed = TRUE, FUN = function(k) {
+      return(SerialRegression(
+        xdata = xdata, ydata = ydata, Lambda = Lambda, pi_list = pi_list,
+        K = ceiling(K / n_cores), tau = tau, seed = as.numeric(paste0(seed, k)), n_cat = n_cat,
+        family = family, implementation = implementation,
+        resampling = resampling, cpss = cpss,
+        PFER_method = PFER_method, PFER_thr = PFER_thr, FDP_thr = FDP_thr,
+        group_x = group_x, group_penalisation = group_penalisation,
+        output_data = output_data, verbose = FALSE, ...
+      ))
+    })
+    future::plan(future::sequential)
+
+    # Combining the outputs from parallel iterations
+    out <- mypar[[1]]
+    if (n_cores > 1) {
+      for (i in 2:length(mypar)) {
+        out <- do.call(Combine, list(stability1 = out, stability2 = mypar[[i]]))
+      }
+    }
+  } else {
+    if (optimisation == "grid_search") {
+      out <- SerialRegression(
+        xdata = xdata, ydata = ydata, Lambda = Lambda, pi_list = pi_list,
+        K = K, tau = tau, seed = seed, n_cat = n_cat,
+        family = family, implementation = implementation,
+        resampling = resampling, cpss = cpss,
+        PFER_method = PFER_method, PFER_thr = PFER_thr, FDP_thr = FDP_thr,
+        group_x = group_x, group_penalisation = group_penalisation,
+        output_data = output_data, verbose = verbose, ...
+      )
+    } else {
+      # Creating the function to be minimised
+      eval_f <- function(x, env) {
+        # Running with a given lambda
+        out_nloptr <- SerialRegression(
+          xdata = xdata, ydata = ydata, Lambda = rbind(x), pi_list = pi_list,
+          K = K, tau = tau, seed = seed, n_cat = n_cat,
+          family = family, implementation = implementation,
+          resampling = resampling, cpss = cpss,
+          PFER_method = PFER_method, PFER_thr = PFER_thr, FDP_thr = FDP_thr,
+          group_x = group_x, group_penalisation = group_penalisation,
+          output_data = output_data, verbose = FALSE, ...
+        )
+        if (any(!is.na(out_nloptr$S))) {
+          score <- max(out_nloptr$S, na.rm = TRUE)
+        } else {
+          score <- -Inf
+        }
+
+        # Storing the visited values
+        out <- get("out", envir = env)
+        out <- Concatenate(out_nloptr, out)
+        assign("out", out, envir = env)
+
+        return(-score)
+      }
+
+      # Defining the nloptr options
+      opts <- list(
+        "algorithm" = "NLOPT_GN_DIRECT_L",
+        "xtol_abs" = 0.1,
+        "ftol_abs" = 0.1,
+        "print_level" = 0,
+        "maxeval" = Lambda_cardinal
+      )
+      if ("opts" %in% names(extra_args)) {
+        for (opts_id in 1:length(extra_args[["opts"]])) {
+          opts[[names(extra_args[["opts"]])[opts_id]]] <- extra_args[["opts"]][[opts_id]]
+        }
+      }
+
+      # Initialising the values to store
+      nloptr_env <- new.env(parent = emptyenv())
+      assign("out", NULL, envir = nloptr_env)
+      nloptr_results <- nloptr::nloptr(
+        x0 = apply(Lambda, 2, max, na.rm = TRUE),
+        eval_f = eval_f,
+        opts = opts,
+        lb = apply(Lambda, 2, min, na.rm = TRUE),
+        ub = apply(Lambda, 2, max, na.rm = TRUE),
+        env = nloptr_env
+      )
+      out <- get("out", envir = nloptr_env)
+      out <- Concatenate(out, order_output = TRUE)
     }
   }
 
@@ -615,7 +706,7 @@ SerialRegression <- function(xdata, ydata = NULL, Lambda, pi_list = seq(0.6, 0.9
     pb <- utils::txtProgressBar(style = 3)
   }
   if (!cpss) {
-    for (k in 1:K) {
+    for (k in seq_len(K)) {
       s <- Resample(data = ydata, family = family, tau = tau, resampling = resampling, ...)
       Xsub <- xdata[s, , drop = FALSE]
       Ysub <- ydata[s, , drop = FALSE]
@@ -656,13 +747,13 @@ SerialRegression <- function(xdata, ydata = NULL, Lambda, pi_list = seq(0.6, 0.9
     bigstab <- matrix(NA, nrow = nrow(Beta), ncol = ncol(Beta))
     colnames(bigstab) <- colnames(Beta)
     rownames(bigstab) <- rownames(Beta)
-    for (i in 1:nrow(Beta)) {
-      for (j in 1:ncol(Beta)) {
+    for (i in seq_len(nrow(Beta))) {
+      for (j in seq_len(ncol(Beta))) {
         bigstab[i, j] <- sum(Beta[i, j, ] != 0, na.rm = TRUE) / sum(!is.na(Beta[i, j, ]))
       }
     }
   } else {
-    for (k in 1:ceiling(K / 2)) {
+    for (k in seq_len(ceiling(K / 2))) {
       s <- Resample(data = ydata, family = family, tau = tau, resampling = resampling, ...)
 
       # First subset
@@ -736,7 +827,7 @@ SerialRegression <- function(xdata, ydata = NULL, Lambda, pi_list = seq(0.6, 0.9
     colnames(bigstab) <- colnames(Beta)
     rownames(bigstab) <- rownames(Beta)
     n_valid <- rep(0, nrow(Beta))
-    for (k in 1:ceiling(K / 2)) {
+    for (k in seq_len(ceiling(K / 2))) {
       A1 <- ifelse(Beta[, , k] != 0, yes = 1, no = 0)
       A2 <- ifelse(Beta[, , ceiling(K / 2) + k] != 0, yes = 1, no = 0)
       A <- A1 + A2
